import java.util.*;

/** An instance represents a blog post as something with 
  * a unique non-negative integer ID number and
  * links to zero or more other blog posts. 
  * Blogs receive consecutive ID numbers, starting at 0.  */
public class Blog {
 
  private static int nextID=0; /** Number of currently existing blogs and
                                 * alsothe next ID number for a blog. */
  
  private int id;         // id number of this Blog
  private int numIn=0;    // number of inlinks to this Blog
  private Vector<Blog> outlinks
    = new Vector<Blog>(); // list of Blogs this Blog links to.
                          // No repeats included, even if a target is
                          //   selected several times.
 
  /** Constructor: a new Blog with the next unused ID number.
    * It does not yet link to any other Blogs*/
  public Blog() {
    id= nextID;
    nextID= nextID+1;
  }
  
  /** Constructor: a new Blog with the next unused ID number
    * that links to the blogs specified in targets.
    * Precondition: targets is not null */ 
  public Blog(Vector<Blog> targets) {
    this();
    
    // link to the Blogs in target
    // inv: targets[0..i-1] have been linked to
    for (int i= 0; i< targets.size(); i= i+1) {
      linkTo(targets.get(i));
    }
    // post: targets[0..targets.size()-1] have been linked to
  }
  
  /** = number of existing Blogs */
  public static int numBlogs() {
    return nextID;
  }
  
  /** = id of this Blog */
  public int getID() {
    return id;
  }
  
  /** = number of links to this Blog */
  public int getNumInLinks() {
   return numIn; 
  }
  
  /** = Blogs to which this Blog links. */
  public Vector<Blog> getOutLinks() {
    return outlinks;
  }
  
  /** increment the number of inlinks to this Blog */
  private void incLinks() {
    numIn= numIn+1;
  }
  
  /** link to Blog target.*/
  public void linkTo(Blog target) {
    // add target only if target not already in outlinks
    if (!outlinks.contains(target)) {
      outlinks.add(target);
      target.incLinks();
    }
  }
    
  /** String representation of this Blog, in the format:
    * "a Blog with <number of inlinks> inlinks and linking to: <list of outlinks>" */
  public String toString() {
    
    // Don't want full listing for Blogs that this Blog links to;
    // just the IDs suffice.
    String outlinkList="";
    
    //inv: outlinkList contains info for the Blogs outlinks[0 .. i-1].
    for (int i=0; i< outlinks.size(); i= i+1) {
      outlinkList= outlinkList+ outlinks.get(i).getID() + ";";
    }
    // post: outLinkList contains info for the Blogs outlinks[0..outlinks.size()-1]
    
    return "Blog " + id + " with " + numIn + " inlink" + (numIn!=1?"s":"") + " and " + 
      (outlinkList==""?"no outlinks":"linking to: " + outlinkList);
  }
}