import java.util.*;
/** An instance simulates multiple trials of various linking policies.
  * Concepts illustrated: 
  * (1) for loops for simulation
  * (2) CS enabling and interacting with the social sciences (interdisciplinary field)
  * (3) use of (uniform) randomness
  * (4) use of abstract classes and methods
  * */
public class BlogNetworkDemo {
 
  /** collection of Blogs to work with.
    * Currently public so we can look at it during lecture */
  public Vector<Blog> blogs= new Vector<Blog>(); 
  
  /** Constructor: a new demo.
    * A network is created containing numBlogs Blogs with
    * link pattern created using LinkPolicy pol.
    * 
    * The first Blog represents the original story/post and thus is 
    * not given any outlinks.
    * Precondition: numBlogs > 1.*/
  public BlogNetworkDemo(int numBlogs, LinkPolicy pol) {
    
    blogs.add(new Blog()); // original blog; no links out
   
    // inv: i-1 blogs have been added to blogs
    // NOTE initialization!
    for (int i= 2; i <= numBlogs; i= i+1) {
      // add a new Blog with links chosen among the existing Blogs
      blogs.add(new Blog(pol.choose(blogs)));
    }
    // post: numBlogs Blogs have been added to blogs
    
  }
  
  /** = Blog in this BlogNetworkDemo with the largest number of links to it.
    * Precondition: blogs is not empty. */
  public Blog mostLinkedTo() {
    
     // find Blog with biggest number of inlinks
    Blog most= blogs.get(0);
    int biggestInDegree= most.getNumInLinks(); 
    
// inv: most is the Blog with the biggest indegree of
    //      candidates[0..i-1], and biggestInDegree is its
    //      indegree
    for (int i= 1; i< blogs.size(); i= i+1) {
      Blog candidate=blogs.get(i);
      int candidateNumInLinks= candidate.getNumInLinks();
      if (candidateNumInLinks > biggestInDegree) {
        most= candidate;
        biggestInDegree= candidateNumInLinks;
      }
        
    }
    // post: most is the Blog with the biggest indegree of
    //  candidates[0..candidates.size()-1], and most has 
    //  biggestInDegree inlinks
    return most; // #@ stub
  }
  
  /** run n iters of creating a Blog network with numBlog blogs
    * according to a copying model with parameter copyp */
  public static void bnDemo(int numBlogs, double copyp, int iters) {
    
    int numFirstWasMax= 0;
    BlogNetworkDemo demo= null; 
    
    // inv: i-1 iterations have been run
    for (int i=1; i<= iters; i= i+1) {
      demo= new BlogNetworkDemo(numBlogs, new CopyLinkPolicy1(copyp));
      if (demo.mostLinkedTo().equals(demo.blogs.get(0))) {
        numFirstWasMax= numFirstWasMax+1;
      }
    }
    // post: iters iterations have been run
    
    System.out.println("Percentage of time the first Blog was the most linked to: " + numFirstWasMax/(double) iters);
    
    // CopyLinkPolicy1 unifpol=new CopyLinkPolicy1(0.0); // DEBUG
    // CopyLinkPolicy1 halfcopypol= new CopyLinkPolicy1(0.5); // DEBUG
    // BlogNetworkDemo unifdemo= new BlogNetworkDemo(numBlogs, unifpol); // DEBUG
    // BlogNetworkDemo halfcopydemo= new BlogNetworkDemo(numBlogs, halfcopypol); // DEBUG
    
    
  }
  
  /** =  information about the blogs */
  public String toString() {
    String out="";
    // inv: out contains info for each of blogs[0..i-1]
    for (int i= 0; i< blogs.size(); i= i+1) {
      out= out + blogs.get(i) + "\n";
    }
    // post: out contains info for each of blogs[0..blogs.size()-1]
    return out;
  }
}