import java.util.*;
/** This abstract class represents a linking policy. It enforces the
  * existence of a method choose(), which returns a Vector
  * of Blogs according to the policy encoded by an instance of a 
  * subclass of Policy. */ 
public abstract class LinkPolicy { 
  
   /** = "coinflip came up heads", given a "coin" with probability
     * p of coming up heads.
     * Precondition: 0.0 <= p <= 1.0 */
   public static boolean coinComesUpHeads(double p) {
     double r= Math.random();
     return r < p;
   }
    
  /** = a Blog chosen from set choices,
     * chosen uniformly at random (u.a.r) (to the degree
     * that one can trust Math.random()).
     * Precondition: choices contains at least one Blog.  */
   public static Blog BlogUAR(Vector<Blog> choices) {
     int index= (int) (Math.random()*choices.size());
     return choices.get(index);
   }

  /** = set (Vector) of Blogs chosen from the Blogs given in 
    * candidates.  
    * Precondition: candidate is not empty*/
  public abstract Vector<Blog> choose(Vector<Blog> candidates) ;
  
   /** method for testing class methods as we go.
     * It is not in testing framework because we have to just look
     * at the output and make a judgment call. */
  public static void someBlogUARTests() {
    Blog x= new Blog();
    Blog y= new Blog();
    Blog z= new Blog(); 
    Vector<Blog> vec= new Vector<Blog>(); 
    vec.add(x);
    vec.add(y); 
    vec.add(z);
    int looplim= 10000;
    System.out.println("Choices vector: " + vec);
    System.out.println("Choices size is: " + vec.size());
 
    // Make looplim random choices of blogs in Vector vec and stor in
    // xcount, ycount, and zcount the number of times blog x, y, and z
    // were chosen, respectively.
    int xcount= 0;
    int ycount= 0;
    int zcount= 0;
    // inv: i-1 random choices have been made
    for (int i= 1; i <= looplim; i= i+1) {
    
      Blog sel= LinkPolicy.BlogUAR(vec);
      if (sel == x) { 
        xcount= xcount+1;
      }
      else if  (sel == y) { 
        ycount= ycount+1;
      }
      else if (sel == z) { 
        zcount= zcount+1;
      }
      // post: looplim random choices have been made
    }
 
    System.out.println("Percentage of time chosen: first: " + xcount/(double)looplim 
                         + " second: " + ycount/(double)looplim 
                         + " third: " + zcount/(double)looplim);
    
    //CopyingLinkPolicy1 pol= new CopyingLinkPolicy1(0.0);
    //System.out.println(pol.choose(vec));
  }
  
  /** See whether ints generated UAR.
    * Not in a testing framework because we want to look 
    * at the output and make a judgment call */
  public static void someUARTests() {
    // This code uses an array. You will learn about arrays in a few weeks.
    int[] counts= {0, 0, 0, 0};
    int size= counts.length;
    int looplim= 10000;
    
    // make looplim selections of random ints from 0..3 and store
    // in each count[i] the number of times i in 0..3 was choen.
    
    // inv: i-1 random choices of integer have been made
    for (int i= 1 ; i <= looplim; i= i + 1) {
        int k= (int) (Math.random()*size);
        counts[k]= counts[k] + 1;
    }
    // post: looplim random choices of integer have been made
    
    // inv: percentages for ints 0 .. j-1 have been printed
    for (int j= 0; j < size; j= j + 1) {
      System.out.print(" count for" + j + ": " + counts[j]/(double)looplim);
    }
    // percentages for ints 0..size-1 have been printed
    System.out.println();
  }
  
}